@@ -434,7 +434,8 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
        for(session_no= 0; session_no < num_sessions; session_no++) {
          tracks= isoburn_toc_session_get_tracks(sessions[session_no],
                                                 &num_tracks);
-         track_count+= num_tracks;
+         if(tracks != NULL)
+           track_count+= num_tracks;
        }
        isoburn_toc_disc_free(disc);
      }
@@ -487,6 +488,8 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
    ret= burn_disc_track_lba_nwa(d, NULL, 0, &lba, &nwa);
    if(ret>0)
      (*o)->nwa= nwa;
+   if((*o)->nwa < (*o)->zero_nwa)
+     (*o)->zero_nwa= 0;
 #endif
 
  }
@@ -674,7 +677,7 @@ void isoburn_disc_erase(struct burn_drive *drive, int fast)
         to zero size and change its state */
      burn_disc_erase(drive, fast);
      o->fabricated_disc_status= burn_disc_get_status(drive);
-     o->nwa= 0;
+     o->nwa= o->zero_nwa= 0;
      goto ex;
    }
    if(o->emulation_mode > 0) { /* might be readonly with emulated sessions */
@@ -1005,6 +1008,8 @@ int isoburn_set_start_byte(struct isoburn *o, off_t value, int flag)
  if(value % caps->start_alignment)
    value+= caps->start_alignment - (value % caps->start_alignment);
  o->nwa= value/2048;
+ if(o->nwa < o->zero_nwa)
+   o->zero_nwa= 0;
  /* If suitable for media alignment, round up to Libisoburn_nwa_alignemenT */
  if((o->nwa % Libisoburn_nwa_alignemenT) &&
      ((Libisoburn_nwa_alignemenT*2048) % caps->start_alignment)==0 )
@@ -1210,7 +1215,6 @@ int isoburn_read_iso_head(struct burn_drive *d, int lba,
  unsigned char *buffer= NULL;
  int ret, info_mode, capacity, role;
  off_t data_count, to_read;
- enum burn_disc_status s;
  struct isoburn *o;
 
  buffer= calloc(1, 64 * 1024);
@@ -1223,7 +1227,6 @@ int isoburn_read_iso_head(struct burn_drive *d, int lba,
    memcpy(buffer, info, 64*1024);
  } else {
    memset(buffer, 0, 64 * 1024);
-   s= isoburn_disc_get_status(d);
    role = burn_drive_get_drive_role(d);
    if (role == 3 || role == 5)
 
