@@ -136,15 +136,33 @@ file itself: libisoburn/libisoburn.h .
 They shall link with -lisofs -lburn -lisoburn or with the .o files emerging
 from building those libraries from their sources.
 
-Applications must use 64 bit off_t, e.g. on 32-bit GNU/Linux by defining
+Applications must use 64 bit off_t.
+E.g. on 32-bit GNU/Linux by defining
   #define _LARGEFILE_SOURCE
   #define _FILE_OFFSET_BITS 64
-or take special precautions to interface with the library by 64 bit integers
-where above .h files prescribe off_t. Not to use 64 bit file i/o will keep 
-the application from producing and processing ISO images of more than 2 GB 
-size.
+The minimum requirement is to interface with the library by 64 bit signed
+integers where libisofs.h or libisoburn.h prescribe off_t.
+Failure to do so may result in surprising malfunction or memory faults.
+
+Application files which include libisofs/libisofs.h or libisoburn/libisoburn.h
+must provide definitions for uint32_t and uint8_t.
+This can be achieved either:
+- by using autotools which will define HAVE_STDINT_H or HAVE_INTTYPES_H 
+  according to its ./configure tests,
+- or by defining the macros HAVE_STDINT_H resp. HAVE_INTTYPES_H according
+  to the local situation,
+- or by appropriately defining uint32_t and uint8_t by other means,
+  e.g. by including inttypes.h before including libisofs.h and libisoburn.h
 
 */
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
+
 
 /* Important: If you add a public API function then add its name to file
                   libisoburn/libisoburn.ver
@@ -223,16 +241,16 @@ void isoburn_version(int *major, int *minor, int *micro);
     @since 0.1.0
 */
 #define isoburn_libisofs_req_major  1
-#define isoburn_libisofs_req_minor  0
-#define isoburn_libisofs_req_micro  6 
+#define isoburn_libisofs_req_minor  1
+#define isoburn_libisofs_req_micro  2 
 
 /** The minimum version of libburn to be used with this version of libisoburn
     at compile time.
     @since 0.1.0
 */
 #define isoburn_libburn_req_major  1
-#define isoburn_libburn_req_minor  0
-#define isoburn_libburn_req_micro  6
+#define isoburn_libburn_req_minor  1
+#define isoburn_libburn_req_micro  0
 
 /** The minimum compile time requirements of libisoburn towards libjte are
     the same as of a suitable libisofs towards libjte.
@@ -286,8 +304,8 @@ int isoburn_libburn_req(int *major, int *minor, int *micro);
     @since 0.1.0
 */
 #define isoburn_header_version_major  1
-#define isoburn_header_version_minor  0
-#define isoburn_header_version_micro  8
+#define isoburn_header_version_minor  1
+#define isoburn_header_version_micro  2
 /** Note:
     Above version numbers are also recorded in configure.ac because libtool
     wants them as parameters at build time.
@@ -460,7 +478,11 @@ int isoburn_drive_set_msgs_submit(struct burn_drive *d,
     @return The status of the drive, or what kind of disc is in it.
             Note: BURN_DISC_UNGRABBED indicates wrong API usage
 */
+#ifdef __cplusplus
+enum burn::burn_disc_status isoburn_disc_get_status(struct burn_drive *drive);
+#else
 enum burn_disc_status isoburn_disc_get_status(struct burn_drive *drive);
+#endif
 
 
 /** Tells whether the media can be treated by isoburn_disc_erase().
@@ -602,7 +624,8 @@ void isoburn_toc_session_get_leadout_entry(struct isoburn_toc_session *s,
     @since 0.1.6
     @param s The session handle
     @param num returns the number of tracks in the array
-    @return the address of the array of track handles
+    @return the address of the array of track handles,
+            NULL if no tracks are registered with session s
 */
 struct isoburn_toc_track **isoburn_toc_session_get_tracks(
                                       struct isoburn_toc_session *s, int *num);
@@ -878,6 +901,14 @@ int isoburn_ropt_set_displacement(struct isoburn_read_opts *o,
 int isoburn_ropt_get_displacement(struct isoburn_read_opts *o,
                                uint32_t *displacement, int *displacement_sign);
 
+/* If you get here because of a compilation error like
+
+       /usr/include/libisoburn/libisoburn.h:895: error:
+       expected declaration specifiers or '...' before 'uint32_t'
+
+   then see above paragraph "Application Constraints" about the definition
+   of uint32_t.
+*/
 
 
 /** After calling function isoburn_read_image() there are informations
