@@ -547,6 +547,12 @@ void *iso_alloc_mem(size_t size, size_t count, int flag);
                 ret= ISO_OUT_OF_MEM; goto ex; \
         } }
 
+#define LIBISO_ALLOC_MEM_VOID(pt, typ, count) { \
+        pt= (typ *) iso_alloc_mem(sizeof(typ), (size_t) (count), 0); \
+        if(pt == NULL) { \
+                goto ex; \
+        } }
+
 #define LIBISO_FREE_MEM(pt) { \
         if(pt != NULL) \
                 free((char *) pt); \
