@@ -345,10 +345,12 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
                              char *filter, FILE *fp, int flag)
 {
  int is_default, no_defaults, i, is_default_id= 0, ret;
- char *line, bspec[SfileadrL + 80], zeros[28];
+ char *line, *bspec= NULL, zeros[28];
  off_t file_size;
  struct stat stbuf;
 
+ Xorriso_alloc_meM(bspec, char, SfileadrL + 80);
+
  no_defaults= flag & 1;
  line= xorriso->result_line;
 
@@ -379,7 +381,7 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
       load_size == 2048 && is_default_id && emul == 0) {
      sprintf(line, "-boot_image isolinux %s\n", bspec);
      Xorriso_status_result(xorriso,filter,fp,flag&2); 
-     return(1);
+     {ret= 1; goto ex;};
    }
 
    file_size= 0;
@@ -393,7 +395,7 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
      Text_shellsafe(bin_path, line, 1);
      strcat(line, "\n");
      Xorriso_status_result(xorriso,filter,fp,flag&2);
-     return(1);
+     {ret= 1; goto ex;};
    }
  }
 
@@ -451,7 +453,10 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2); 
  
- return(1); 
+ ret= 1; 
+ex:;
+ Xorriso_free_meM(bspec);
+ return(ret); 
 }
 
 
@@ -524,7 +529,7 @@ int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
 {
  int ret, i, num_boots, hflag;
  int bin_path_in_use= 0, is_default, no_defaults;
- char sfe[5*SfileadrL], path[SfileadrL], *form= "any", *line, *hpt;
+ char *path= NULL, *form= "any", *line, *hpt;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
  IsoImage *image= NULL;
@@ -534,6 +539,7 @@ int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
  enum eltorito_boot_media_type media_type;
  unsigned char id_string[29], sel_crit[21];
 
+ Xorriso_alloc_meM(path, char, SfileadrL);
  line= xorriso->result_line;
  no_defaults= flag & 1;
 
@@ -573,8 +579,9 @@ no_image:;
    bin_path_in_use= 1;
  if(xorriso->boot_image_cat_path[0] && bin_path_in_use) {
    is_default= 0;
-   sprintf(line,"-boot_image %s cat_path=%s\n",
-           form, Text_shellsafe(xorriso->boot_image_cat_path, sfe, 0));
+   sprintf(line,"-boot_image %s cat_path=", form);
+   Text_shellsafe(xorriso->boot_image_cat_path, line, 1);
+   strcat(line, "\n");
    if(!(is_default && no_defaults))
      Xorriso_status_result(xorriso,filter,fp,flag&2);
  }
@@ -666,6 +673,9 @@ ex:
    free(boots);
  if(bootnodes != NULL)
    free(bootnodes);
+ if(image != NULL)
+   iso_image_unref(image);
+ Xorriso_free_meM(path);
  return(ret);
 }
 
@@ -694,8 +704,8 @@ int Xorriso_show_boot_info(struct XorrisO *xorriso, int flag)
  int ret, bin_path_valid= 0,has_isolinux_mbr= 0, i, num_boots;
  unsigned int mbr_lba= 0;
  off_t lb0_count;
- char *respt, sfe[5*SfileadrL], path[SfileadrL];
- unsigned char lb0[2048];
+ char *respt, *path;
+ unsigned char *lb0= NULL;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
  IsoImage *image= NULL;
@@ -703,6 +713,9 @@ int Xorriso_show_boot_info(struct XorrisO *xorriso, int flag)
  IsoFile *bootimg_node, **bootnodes = NULL;
  IsoBoot *bootcat_node;
 
+ Xorriso_alloc_meM(path, char, SfileadrL);
+ Xorriso_alloc_meM(lb0, unsigned char, 2048);
+
  respt= xorriso->result_line;
 
  if(xorriso->boot_count > 0) {
@@ -757,7 +770,7 @@ no_boot:;
        has_isolinux_mbr= 0;
      mbr_lba= lb0[432] | (lb0[433] << 8) | (lb0[434] << 16) | (lb0[435] << 24);
      mbr_lba/= 4;
-     if(mbr_lba != xorriso->loaded_boot_bin_lba)
+     if(mbr_lba != (unsigned int) xorriso->loaded_boot_bin_lba)
        has_isolinux_mbr= 0;
      if(has_isolinux_mbr) {
        for(i= 0; i < 426; i++)
@@ -782,20 +795,24 @@ no_boot:;
  if(flag & 2)
    {ret= 1; goto ex;}
 
- if(xorriso->loaded_boot_cat_path[0])
-   sprintf(respt, "Boot catalog : %s\n",
-           Text_shellsafe(xorriso->loaded_boot_cat_path, sfe, 0));
- else
+ if(xorriso->loaded_boot_cat_path[0]) {
+   sprintf(respt, "Boot catalog : ");
+   Text_shellsafe(xorriso->loaded_boot_cat_path, respt, 1);
+   strcat(respt, "\n");
+ } else {
    sprintf(respt, "Boot catalog : -not-found-at-load-time-\n");
+ }
  Xorriso_toc_line(xorriso, flag & 8);
 
- if(bin_path_valid)
-   sprintf(respt, "Boot image   : %s", Text_shellsafe(path, sfe, 0));
- else if(xorriso->loaded_boot_bin_lba <= 0)
+ if(bin_path_valid) {
+   sprintf(respt, "Boot image   : ");
+   Text_shellsafe(path, respt, 1);
+ } else if(xorriso->loaded_boot_bin_lba <= 0) {
    sprintf(respt, "Boot image   : -not-found-at-load-time-");
- else
+ } else {
    sprintf(respt, "Boot image   : -not-found-any-more-by-lba=%d",
            xorriso->loaded_boot_bin_lba);
+ }
  Xorriso__append_boot_params(respt, bootimg, 0);
  strcat(respt, "\n");
  Xorriso_toc_line(xorriso, flag & 8);
@@ -803,9 +820,10 @@ no_boot:;
  if(num_boots > 1) {
    for(i= 1; i < num_boots; i++) {
      ret= Xorriso_path_from_node(xorriso, (IsoNode *) bootnodes[i], path, 0);
-     if(ret > 0)
-       sprintf(respt, "Boot image   : %s", Text_shellsafe(path, sfe, 0));
-     else
+     if(ret > 0) {
+       sprintf(respt, "Boot image   : ");
+       Text_shellsafe(path, respt, 1);
+     } else
        sprintf(respt, "Boot image   : -not-found-any-more-");
      Xorriso__append_boot_params(respt, boots[i], 0);
      strcat(respt, "\n");
@@ -816,10 +834,12 @@ no_boot:;
 ex:;
  if(boots != NULL)
    free(boots);
- if(bootnodes != NULL);
+ if(bootnodes != NULL)
    free(bootnodes);
  if(image != NULL)
    iso_image_unref(image); /* release obtained reference */
+ Xorriso_free_meM(path);
+ Xorriso_free_meM(lb0);
  return(ret);
 } 
 
@@ -889,13 +909,17 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
 {
  int ret, lba, track, session, params_flag= 0, is_safe= 0, is_extra_drive= 0;
  int give_up= 0, mount_chardev= 0, status, aquire_flag= 0;
- char volid[33], *devadr, mount_command[SfileadrL], adr_data[163], *adr_pt;
- char *dev_path, libburn_adr[BURN_DRIVE_ADR_LEN + SfileadrL];
- char sfe[5 * SfileadrL], *dpt, *sysname= "";
+ char volid[33], *devadr, *mount_command= NULL, *adr_data= NULL, *adr_pt;
+ char *dev_path, *libburn_adr= NULL;
+ char *dpt, *sysname= "";
  struct stat stbuf;
  struct burn_drive_info *dinfo= NULL;
  struct burn_drive *drive= NULL;
 
+ Xorriso_alloc_meM(mount_command, char, SfileadrL);
+ Xorriso_alloc_meM(adr_data, char, 163);
+ Xorriso_alloc_meM(libburn_adr, char, BURN_DRIVE_ADR_LEN + SfileadrL);
+
  devadr= dev;
  adr_pt= adr_value;
  if(strcmp(dev, "indev") == 0) {
@@ -906,6 +930,8 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    dev_path= devadr= xorriso->indev;
    if(strncmp(dev_path, "stdio:", 6) == 0)
      dev_path+= 6;
+   else if(strncmp(dev_path, "mmc:", 4) == 0)
+     dev_path+= 4;
    if(xorriso->in_drive_handle == xorriso->out_drive_handle)
      give_up= 3;
    else
@@ -919,6 +945,8 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    dev_path= devadr= xorriso->outdev;
    if(strncmp(dev_path, "stdio:", 6) == 0)
      dev_path+= 6;
+   else if(strncmp(dev_path, "mmc:", 4) == 0)
+     dev_path+= 4;
    if(xorriso->in_drive_handle == xorriso->out_drive_handle)
      give_up= 3;
    else
@@ -928,12 +956,14 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    dev_path= dev;
    if(strncmp(dev_path, "stdio:", 6) == 0)
      dev_path+= 6;
+   else if(strncmp(dev_path, "mmc:", 4) == 0)
+     dev_path+= 4;
 
    /* do only accept regular files and block devices */
    ret= stat(dev_path, &stbuf);
    if(ret == -1) {
-     sprintf(xorriso->info_text, "Cannot determine properties of file %s",
-             Text_shellsafe(dev_path, sfe, 0));
+     sprintf(xorriso->info_text, "Cannot determine properties of file ");
+     Text_shellsafe(dev_path, xorriso->info_text, 1);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      ret= 0; goto ex;
    }
@@ -943,20 +973,25 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    if(!(S_ISREG(stbuf.st_mode) || (S_ISBLK(stbuf.st_mode) && !mount_chardev)
         || (S_ISCHR(stbuf.st_mode) && !mount_chardev))) {
      sprintf(xorriso->info_text,
-             "File object is not suitable as mount device: %s",
-             Text_shellsafe(dev_path, sfe, 0));
+             "File object is not suitable as mount device: ");
+     Text_shellsafe(dev_path, xorriso->info_text, 1);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      ret= 0; goto ex;
    }
 
    /* Aquire drive as direct libburn address or via stdio: prefix */
-   ret= burn_drive_convert_fs_adr(dev, libburn_adr);
+   if(strncmp(dev, "mmc:", 4) == 0)
+     ret= burn_drive_convert_fs_adr(dev + 4, libburn_adr);
+   else
+     ret= burn_drive_convert_fs_adr(dev, libburn_adr);
    Xorriso_process_msg_queues(xorriso,0);
    if(ret < 0)
      {ret= -1; goto ex;}
-   if(ret == 0 && strncmp(dev, "stdio:", 6) != 0)
+   if(ret == 0 && strncmp(dev, "stdio:", 6) != 0 &&
+      strncmp(dev, "mmc:", 4) != 0)
      sprintf(libburn_adr, "stdio:%s", dev);
-   burn_preset_device_open(xorriso->drives_exclusive, 0, 0);
+   burn_preset_device_open(
+           xorriso->drives_exclusive && !(xorriso->mount_opts_flag & 1), 0, 0);
    aquire_flag= 1;
    if((xorriso->toc_emulation_flag & 2) && adr_mode == 3)
      aquire_flag|= 16;
@@ -988,6 +1023,8 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
  }
  if(strstr(devadr, "stdio:") == devadr)
    devadr+= 6;
+ if(strstr(devadr, "mmc:") == devadr)
+   devadr+= 4;
  ret= Xorriso_make_mount_cmd(xorriso, cmd, lba, track, session, volid, devadr,
                              mount_command, flag & (2 | 4));
  if(ret <= 0)
@@ -1006,8 +1043,9 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
  }
  Xorriso_process_msg_queues(xorriso,0);
 
- sprintf(xorriso->info_text, "Volume id    : %s\n",
-                             Text_shellsafe(volid, sfe, 0));
+ sprintf(xorriso->info_text, "Volume id    : ");
+ Text_shellsafe(volid, xorriso->info_text, 1);
+ strcat(xorriso->info_text, "\n");
  Xorriso_info(xorriso, 0);
  if(flag & 1) {
    sprintf(xorriso->result_line, "%s\n", mount_command);
@@ -1030,15 +1068,16 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        ret= 0; goto ex;
      }
-     sprintf(xorriso->info_text, "\nMounted session %d of device %s",
-                                 session, Text_shellsafe(dev_path, sfe, 0));
+     sprintf(xorriso->info_text, "\nMounted session %d of device ", session);
+     Text_shellsafe(dev_path, xorriso->info_text, 1);
      dpt= strchr(cmd, ':');
      if(dpt == NULL)
        dpt= cmd ;
      else
        dpt++;
-     sprintf(xorriso->info_text + strlen(xorriso->info_text),
-             " as directory %s\n", Text_shellsafe(dpt, sfe, 0));
+     sprintf(xorriso->info_text + strlen(xorriso->info_text), " as directory ");
+     Text_shellsafe(dpt, xorriso->info_text, 1);
+     strcat(xorriso->info_text, "\n");
      Xorriso_info(xorriso, 0);
    }
  }
@@ -1049,6 +1088,9 @@ ex:;
    burn_drive_info_free(dinfo);
    Xorriso_process_msg_queues(xorriso,0);
  }
+ Xorriso_free_meM(mount_command);
+ Xorriso_free_meM(adr_data);
+ Xorriso_free_meM(libburn_adr);
  return(ret);
 }
 
