@@ -78,22 +78,37 @@ int Xorriso_abort(struct XorrisO *xorriso, int flag)
 
 
 /* @param flag bit0= asynchronous handling (else catch thread, wait, and exit)
+               bit1= dealing with MMC drive in critical state
+                     behavior 2 -> behavior 1
 */
 int Xorriso_set_signal_handling(struct XorrisO *xorriso, int flag)
 {
  char *handler_prefix= NULL;
+ int behavior, mode;
 
- if(Xorriso__get_signal_behavior(0) != 1)
+ behavior= Xorriso__get_signal_behavior(0);
+ if(behavior == 0)
    return(2);
+ if(behavior == 2 && !(flag & 2))
+   mode= 1;
+ else if(behavior == 3)
+   mode= 2;
+ else
+   mode= (flag & 1) * 0x30;
  handler_prefix= calloc(strlen(xorriso->progname)+3+1, 1);
  if(handler_prefix==NULL) {
    sprintf(xorriso->info_text,
-           "Cannot allocate memory for for setting signal handler");
+           "Cannot allocate memory for setting signal handler");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FATAL", 0);
    return(-1);
  }
+
+ /* <<< */
+ sprintf(xorriso->info_text, "burn_set_signal_handling(%d)", mode);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+
  sprintf(handler_prefix, "%s : ", xorriso->progname);
- burn_set_signal_handling(handler_prefix, NULL, (flag & 1) * 0x30);
+ burn_set_signal_handling(handler_prefix, NULL, mode);
  free(handler_prefix);
  return(1);
 }
@@ -369,9 +384,11 @@ int Xorriso_report_iso_error(struct XorrisO *xorriso, char *victim,
 {
  int error_code, iso_sev, min_sev, ret;
  char *sev_text_pt, *msg_text_pt= NULL;
- char sfe[6*SfileadrL];
+ char *sfe= NULL;
  static int sorry_sev= -1;
 
+ Xorriso_alloc_meM(sfe, char, 6 * SfileadrL);
+
  if(sorry_sev<0)
    Xorriso__text_to_sev("SORRY", &sorry_sev, 0);
 
@@ -401,6 +418,8 @@ int Xorriso_report_iso_error(struct XorrisO *xorriso, char *victim,
    Text_shellsafe(victim, sfe+strlen(sfe), 0);
  }
  ret= Xorriso_msgs_submit(xorriso, error_code, sfe, os_errno, sev_text_pt, 4);
+ex:;
+ Xorriso_free_meM(sfe);
  return(ret);
 }
 
@@ -415,7 +434,7 @@ int Xorriso_get_local_charset(struct XorrisO *xorriso, char **name, int flag)
 int Xorriso_set_local_charset(struct XorrisO *xorriso, char *name, int flag)
 {
  int ret;
- char *nl_charset, sfe[5 * SfileadrL];
+ char *nl_charset;
  iconv_t iconv_ret= (iconv_t) -1;
 
  nl_charset= nl_langinfo(CODESET);
@@ -433,13 +452,13 @@ int Xorriso_set_local_charset(struct XorrisO *xorriso, char *name, int flag)
  if(ret <= 0) {
 cannot:;
    sprintf(xorriso->info_text,
-           "-local_charset: Cannot assume as local character set: %s",
-           Text_shellsafe(name, sfe, 0));
+           "-local_charset: Cannot assume as local character set: ");
+   Text_shellsafe(name, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
    return(0);
  }
- sprintf(xorriso->info_text, "Local character set is now assumed as: %s",
-         Text_shellsafe(name, sfe, 0));
+ sprintf(xorriso->info_text, "Local character set is now assumed as: ");
+ Text_shellsafe(name, xorriso->info_text, 1);
  Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
  return(1);
 }
