@@ -572,13 +572,12 @@ done:;
 
 time_t Decode_timestring(char *code, time_t *date, int flag)
 {
- char *cpt,scale_chr;
+ char scale_chr;
  double value,seconds;
  struct tm result_tm;
  int seconds_valid= 0;
 
  *date= 0;
- cpt= code;
  if(code[0]=='-' || code[0]=='+' || code[0]=='=' || code[0]=='@'){
    if(code[1]==0)
      return(0);
@@ -873,7 +872,7 @@ int Sregex_resolve_var(char *form, char *vars[][2], int num_vars,
                        char *start, char *end, char *esc,
                        char *result, int result_size, int flag)
 {
- int l_e, l_v, l_s, l_esc, i, start_equals_esc;
+ int l_e, l_s, l_esc, i, start_equals_esc;
  char *rpt, *wpt, *spt, *npt, *ept;
 
  if(start[0] == 0) /* It is not allowed to have no start marker */
@@ -947,7 +946,6 @@ int Sregex_resolve_var(char *form, char *vars[][2], int num_vars,
    }
    if(i < num_vars) {
      /* substitute found variable */
-     l_v= strlen(vars[i][0]);
      if((wpt - result) + (int) strlen(vars[i][1]) >= result_size)
        return(0);
      strcpy(wpt, vars[i][1]);
