@@ -530,7 +530,7 @@ int Xorriso_option_biblio_file(struct XorrisO *xorriso, char *name, int flag)
 int Xorriso_option_blank(struct XorrisO *xorriso, char *mode, int flag)
 {
  char *cmd= "-blank";
- int aq_ret, ret, aq_flag= 2, mode_flag= 0, as_needed= 0, idx;
+ int aq_ret, ret, mode_flag= 0, as_needed= 0, idx;
  off_t size= 0;
 
  if(flag&1)
@@ -548,7 +548,6 @@ int Xorriso_option_blank(struct XorrisO *xorriso, char *mode, int flag)
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      return(0);
    }
-   aq_flag= 3;
  }
  ret= Xorriso_reassure(xorriso, cmd,
                        "possibly make unreadable data on outdev", 0);
