@@ -449,7 +449,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
  static char arg0_commands[][40]= {
     "ban_stdio_write","close_filter_list","commit","devices","end",
     "for_backup", "help",
-    "list_formats","no_rc","print_size","pvd_info","pwd","pwdi","pwdx",
+    "list_formats","list_speeds",
+    "no_rc","print_size","pvd_info","pwd","pwdi","pwdx",
     "read_mkisofsrc","rollback","rollback_end",
     "tell_media_space","toc","version",
     ""
@@ -581,7 +582,7 @@ return:
 */
 {
  int ret, was_dashed, end_ret, num1, num2, cmd_data_size= 2 * SfileadrL;
- int mem_idx;
+ int mem_idx, arg_count, i;
  char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
  
  Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
@@ -610,6 +611,22 @@ next_command:;
    }
  }
 
+ ret= Xorriso_count_args(xorriso, argc - *idx, argv + *idx, &arg_count, 1);
+ if((ret == 1 || ret == 2) &&
+    strcmp(argv[*idx], xorriso->list_delimiter) != 0) {
+   sprintf(xorriso->info_text, "Command:  %s", argv[*idx]);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+   for(i= 1; i <= arg_count && *idx + i < argc; i++) {
+     sprintf(xorriso->info_text, "Argument:    %s", argv[*idx + i]);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+   }
+   if(*idx + arg_count >= argc) {
+     sprintf(xorriso->info_text, "Missing arguments: %d",
+             *idx + arg_count + 1 - argc);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+   }
+ }
+
  xorriso->prepended_wd= 0;
  xorriso->request_to_abort= xorriso->request_not_to_ask= 0;
  Xorriso_set_problem_status(xorriso, "", 0);
@@ -622,7 +639,6 @@ next_command:;
    goto add_plain_argument;
  }
  was_dashed= 0;
-
  ret= Xorriso_normalize_command(xorriso, original_cmd, -1,
                                 cmd_data, cmd_data_size, &cmd, 0);
  if(ret<0)
@@ -1010,6 +1026,9 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_list_profiles(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"list_speeds")==0) {
+   ret= Xorriso_option_list_speeds(xorriso, 0);
+
  } else if(strcmp(cmd,"load")==0) {
    (*idx)+= 2;
    ret= Xorriso_option_load(xorriso, arg1, arg2, 0);
@@ -1506,7 +1525,7 @@ ex:;
 
 int Xorriso_dialog(struct XorrisO *xorriso, int flag)
 {
- int ret,first_round, line_size= 2 * SfileadrL;
+ int ret, line_size= 2 * SfileadrL;
  char *line= NULL;
 
  Xorriso_alloc_meM(line, char, line_size);
@@ -1516,7 +1535,7 @@ int Xorriso_dialog(struct XorrisO *xorriso, int flag)
  if(xorriso->abort_on_is_default)
    Xorriso_option_abort_on(xorriso, "NEVER", 0);
  xorriso->is_dialog= 1;
- for(first_round= 1;1;first_round= 0) {
+ while(1) {
    if(xorriso->pending_option[0]!=0) {
      Xorriso_mark(xorriso,0);
      strcpy(line,xorriso->pending_option);
