@@ -1695,7 +1695,7 @@ int Xorriso_blank_as_needed(struct XorrisO *xorriso, int flag)
 int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
                        char *track_source, off_t tsize, int flag)
 {
- int ret, fd, unpredicted_size, profile_number, is_cd= 0, dummy, nwa= -1;
+ int ret, fd, profile_number, is_cd= 0, dummy, nwa= -1;
  int isosize= -1, do_isosize, is_bd= 0, signal_mode;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
@@ -1755,8 +1755,6 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
        if((stbuf.st_mode&S_IFMT)==S_IFREG)
          fixed_size= stbuf.st_size;
  }
- if(fixed_size==0)
-   unpredicted_size= 1;
 
  if(fd>=0)
    data_src= burn_fd_source_new(fd, -1, fixed_size);
