@@ -60,8 +60,8 @@ struct XorrisO;
     @since 0.5.8
 */
 #define Xorriso_header_version_majoR  1
-#define Xorriso_header_version_minoR  0
-#define Xorriso_header_version_micrO  8
+#define Xorriso_header_version_minoR  1
+#define Xorriso_header_version_micrO  2
 
 
 /** Eventually something like ".pl01" to indicate a bug fix. Normally empty.
@@ -136,6 +136,10 @@ char *Xorriso__get_patch_level_text(int flag);
                    1= use libburn signal handler. Most time with action
                       0. During writing, formatting, blanking: 0x30.
                       Only usable with a single xorriso object.
+                   2= Enable system default reaction on all signals
+                      @since 1.0.9
+                   3= Try to ignore nearly all signals
+                      @since 1.0.9
    @param flag     unused yet, submit 0
    @return         <= 0 is error, >0 is success
 */
@@ -227,23 +231,29 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag);
 /* --------------------- Command Interpreters ------------------- */
 
 
-/* This special interpreter is eventually called between Xorriso_new() and
+/* This special interpreter may be called between Xorriso_new() and
    Xorriso_startup_libraries(). It interprets certain commands which shall
    get into effect before the libraries get initialized:
-     -abort_on , -report_about , -return_with , -list_delimiter
+     -abort_on , -report_about , -return_with , -list_delimiter ,
+     -scsi_log , -signal_handling
    Some commands get executed only if they are the only command in argv:
      -prog_help , -help , -no_rc
    Some get examined for the need to redirect stdout messages
      -dev , -outdev , -indev , -as 
+   Commands -list_delimiter and -add_plainly get into effect during this
+   call. But their setting at begin of the call gets restored before the
+   call returns.
    @param xorriso The context object in which to perform the commands.
    @param argc    Number of arguments.
    @param argv    The arguments. argv[0] contains the program name.
                   argv[1] to argv[argc-1] contain commands and parameters.
    @param idx     Argument cursor. When this function is called, *idx must
-                  be at least 1, argv[*idx] must be to a command.
+                  be at least 1, argv[*idx] must be a command.
                   *idx will iterate over commands and parameters until this
                   function aborts or until argc is reached.
    @param flag    bit0= do not interpret argv[1]
+                  bit1= produce FAILURE events on unknown commands
+                        @since 1.1.0
    @return        <0 error
                    0 end program
                    1 ok, go on 
@@ -263,7 +273,7 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
 int Xorriso_read_rc(struct XorrisO *xorriso, int flag);
 
 
-/* Check whether program arguments shall be backslash decoded and eventually
+/* Check whether program arguments shall be backslash decoded. If so, then
    replace *argv by a new argument vector. The old one will not be freed
    by this call. If it is dynamic memory then you need to keep a copy of
    the pointer and free it yourself after this call.
@@ -323,7 +333,8 @@ int Xorriso_interpreter(struct XorrisO *xorriso,
 int Xorriso_execute_option(struct XorrisO *xorriso, char *line, int flag);
 
 
-/* Enter xorriso command line dialog mode, eventually using libreadline.
+/* Enter xorriso command line dialog mode, using libreadline if configured
+   at build time and not disabled at run time.
    This call returns immediately if not option -dialog "on" was performed
    before.
    @param xorriso The context object in which to perform the commands.
@@ -714,6 +725,11 @@ int Xorriso_option_clone(struct XorrisO *xorriso, char *origin, char *dest,
 /* Option -close "on"|"off" */
 int Xorriso_option_close(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Option -close_damaged */
+/* @since 1.1.0 */
+int Xorriso_option_close_damaged(struct XorrisO *xorriso, char *mode,
+                                 int flag);
+
 /* Option -close_filter_list */
 int Xorriso_option_close_filter_list(struct XorrisO *xorriso, int flag);
 
@@ -833,7 +849,7 @@ int Xorriso_option_external_filter(struct XorrisO *xorriso,
 /* Options -extract , -extract_single */
 /* @param flag bit0=do not report the restored item
                bit1=do not reset pacifier, no final pacifier message
-               bit5= -extract_single: eventually do not insert directory tree
+               bit5= -extract_single: do not insert directory tree
 */
 int Xorriso_option_extract(struct XorrisO *xorriso, char *disk_path,
                        char *iso_path, int flag);
@@ -910,6 +926,9 @@ int Xorriso_option_list_formats(struct XorrisO *xorriso, int flag);
 int Xorriso_option_list_profiles(struct XorrisO *xorriso, char *which,
                                  int flag);
 
+/* Option -list_speeds */
+int Xorriso_option_list_speeds(struct XorrisO *xorriso, int flag);
+
 /* Option -load session|track|sbsector value */
 /* @param flag bit0= with adr_mode sbsector: adr_value is possibly 16 too high
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -1147,6 +1166,12 @@ int Xorriso_option_setfattr_listi(struct XorrisO *xorriso, char *path,
 int Xorriso_option_set_filter(struct XorrisO *xorriso, char *name,
                               int argc, char **argv, int *idx, int flag);
 
+/* Option -signal_handling */
+/* @param flag bit0= do not yet install the eventual handler
+*/
+int Xorriso_option_signal_handling(struct XorrisO *xorriso, char *mode,
+                                   int flag);
+
 /* Option -speed */
 int Xorriso_option_speed(struct XorrisO *xorriso, char *speed, int flag);
 
